#!/usr/bin/lua
require "lunit"
log = require "syslog"
lfs = require "lfs"
json = require "cjson"

module("processor", package.seeall, lunit.testcase)

function setup()
	-- Setup before each test
	local currentdir = lfs.currentdir()
	package.path = package.path..";"..currentdir.."/share/?.lua;"..currentdir.."/etc/rules/?.lua;"..currentdir.."/etc/rules/custom/?.lua;"..currentdir.."/etc/?.lua;"
	require "config"

	-- Make stdout print to variable "nonstdout"
	copy = {
		write = function(self, ...) nonstdout = ... end,
	}
	copy.__index = copy
	io.stdout = setmetatable({mode = "stdout"}, copy)
end


-- Maybe this test doesn't belong here BUT at least it is not released to production.
function test_no_debug_true_commited()
	assert_false(config.debug, "Do not commit config.debug = true!!!")
end

----------------------------------------------
-- Test for startup errorrs
-- This also tests that livestatus socket is initiated and config is set.
----------------------------------------------
function test_for_startup_errors()
	-- Hi-jack args
	arg[1] = nil
	local s, processor =  pcall(loadfile, "bin/synergy")
	local output = processor()
	assert_equal("Nothing to do", output, "Expected processor to start up with no errors and return \"Nothing to do\", got "..tostring(output))
end

----------------------------------------------
-- Test that nothing crashes when spool is empty
----------------------------------------------
function test_empty_rule_spool()
	-- Hi-jack args
	arg[1] = "--process_spool"
	lfs.mkdir("var/")
	-- Set new spool dir
	config.spool_dir = "var/"
	local s, processor, m =  pcall(loadfile, "bin/synergy")
	assert(s, "Expected no errors with processor, got errors")
	local status = processor()
	assert_true(status, "Expected function to return true, no such luck I'm afraid")
	lfs.rmdir("var/")
	local st, msg = lfs.chdir("var/")
	assert_nil(st, "Expected directory to be removed, but it wasn't")
end

----------------------------------------------
-- Test getting a full result
----------------------------------------------
function test_execute()
	arg[1] = "--execute"
	local s, processor = pcall(loadfile, "bin/synergy")
	-- Checks no errors were thrown
	assert_true(s, "Execute failed. Got: "..tostring(processor))
	-- Checks that a result was returned that was not false or nil
	processor()
	assert(nonstdout, "Execute failed. Got: "..tostring(nonstdout))
	local s, decoded_result = pcall(json.decode, nonstdout)
	assert_true(s, "Decoding of json failed.")
	assert_table(decoded_result, "No proper json string was returned")
end

----------------------------------------------
-- Test getting a top level result
----------------------------------------------
function test_top()
	arg[1] = "--top"
	local s, processor = pcall(loadfile, "bin/synergy")
	-- Checks no errors were thrown
	assert_true(s, "Top failed. Got: "..tostring(processor))
	-- Checks that a result was returned that was not false or nil
	processor()
	assert(nonstdout, "Top failed. Got: "..tostring(nonstdout))
	local s, decoded_result = pcall(json.decode, nonstdout)
	assert_true(s, "Decoding of json failed.")
	assert_table(decoded_result, "No proper json string was returned")
end

function teardown()
	-- Teardown after each test
end
